/*
Programa honen bidez, OUTDOOR robota urrunetik kontrola daiteke, Androideko RC Car aplikazioa erabiliz. 
Robotak ultrasoinu sentsorea du aurrean eta oztoporen batera 50cm-ra gerturatzen denean, automatikoki robota
geratu egingo da. Kasu horretan ezin izango da inolaz ere aurrera egin, eta beraz, mugikorretik atzerako aginduak
soilik onartuko dira. Bestalde, unero robota bere egoeraren berri emateko mezuak bidaltzen dizkio echo zerbitzariari.
Honetarako, robota eta echo zerbitzaria wifi sare berera konektatu behar dira. GPS moduluaren bidez, uneko posizioa
kalkulatzen du robotak

OSAGAIAK: 

 - Motorrak
 - Bluetooth
 - Ultrasoinu sentsorea
 - Wifi shield
 - GPS/GSM/GPRS modulua
 
 Android aplikazioa: Bluetooth RC Car:   https://play.google.com/store/apps/details?id=braulio.calle.bluetoothRCcontroller&hl=es
*/

#include <SoftwareSerial.h>
#include <Wifi.h>
#include <Motorrak.h>
#include <GPS.h>


/*PARAMETRO KONFIGURAGARRIAK*/

//GPS parametroa
int irakurketa_kop=30;

//Wifi parametroak
String AP_izena="nireAP"; //Konektatu nahi dugun AParen izena
String AP_pass="123456789"; //AParen pasahitza
String zerb_helb="192.168.43.211";//echo zerbitzaria dagoen makinaren IP helbidea
String zerb_portua="50001"; //echo zerbitzariaren portua

//Bluetooth
int RxD=10;
int TxD=11;
SoftwareSerial BTSerial(RxD, TxD);

//Motorren objektua sortu
Motorrak motorrak(4,5,7,6,0);

//Wifi objektua sortu
Wifi wifi;

//GPS
GPS GPS(3,9,13);

//Robota arriskuan dagoen edo ez adierazteko aldagaia
volatile boolean ARRISKUAN = false;


  /*...KONFIGURAZIO OROKORRA...*/

void setup()
{
  //BLuetooth konfigurazioa
  BTSerial.begin(9600); //Komunikazio abiadura finkatu
  BTSerial.flush(); //Portu serieko memoria garbitu
  delay(500);
  
  //Serieko komunikazioa abiarazi
  Serial.begin(115200);
 
  // Wifia konfiguratu 
  konfiguratu_wifi();
  delay(500); 
    
  //Ultrasoinuko etenenen zerbitzu errutina
  attachInterrupt(0, oztopoa, CHANGE); //Ultrasoinuaren TRIG hankan (pin 2) aldaketa sumatzean oztopoa() zerbitzu errutina exekutatu
}



 /// *** MAIN *** ///
 
void loop()
{
      
      if (BTSerial.available()) // Itxaron bluetooth bidez komandoren bat jaso arte
      {       
          char komandoa = BTSerial.read(); // Irakurri jasotako komandoa
          BTSerial.flush(); //Cache memoria garbitu
       
         
          //Komandoa prozesatu
          switch(komandoa)
          {
             case 'F': //Aurrera
                 if(!ARRISKUAN){
                   motorrak.aurrera();
                 }    
                 else{
                    wifi.bidali("Oztopoa gertuegi. Ezin da aurrera egin");
                  }
                 break;
             
              case 'B': //Atzera
                  motorrak.atzera();
                  break;
         
              case 'L': //Ezkerrera
                  motorrak.ezkerrera();
                  break;
         
              case 'R': //Eskuinera
                  motorrak.eskuinera();
                  break;
               
              case 'I': //Aurrera-Eskuinera
                  if(!ARRISKUAN){
                    motorrak.aurreraEskuinera();
                  }
                  else{
                    wifi.bidali("Oztopoa gertuegi. Ezin da aurrera egin");
                  }
                  break;
               
              case 'G': //Aurrera-Ezkerrera
                  if(!ARRISKUAN){
                    motorrak.aurreraEzkerrera();
                  }
                  else{
                    wifi.bidali("Oztopoa gertuegi. Ezin da aurrera egin");
                  }  
                  break;
               
              case 'H': //Atzera-Ezkerrera
                   motorrak.atzeraEzkerrera();
                   break;
               
              case 'J': //Atzera-Eskuinera
                   motorrak.atzeraEskuinera();
                   break;
                        
              //Aldatu abiadurak                     
              case '0': 
                   motorrak.aldatuAbiadura(0);
                   break;
              
              case '1': 
                   motorrak.aldatuAbiadura(50);
                   break;   
                 
              case '2': 
                   motorrak.aldatuAbiadura(100);
                   break;   
               
              case '3': 
                   motorrak.aldatuAbiadura(150);
                   break;
               
               case '4': 
                   motorrak.aldatuAbiadura(175);
                   break;
          
               
               case '5':
                   motorrak.aldatuAbiadura(200);
                   break;
                  
               case '6':
                   motorrak.aldatuAbiadura(210);
                   break;
         
               case '7':
                  motorrak.aldatuAbiadura(220);
                  break;       
                  
               case '8':
                   motorrak.aldatuAbiadura(230);
                   break;
                   
               case '9':
                   motorrak.aldatuAbiadura(240);
                   break;
                   
               case 'q':
                   motorrak.aldatuAbiadura(255);
                   break;
                                 
               case 'S': //Gelditu
                   motorrak.gelditu();
                   break;
              
               case 'D'://Amaitu
                   motorrak.gelditu();
                   delay(1000);
                   wifi.bidali("Programa amaitu da");
                   exit(0);  
                   break;
        
               case 'X': //Lokalizatu
                   wifi.bidali("Robota kokatzen...");
                   kokatu();
                   break;
                 
               case 'x': //Lokalizatu
                   wifi.bidali("Robota kokatzen...");
                   kokatu();
                   break;
              
               default:
                   wifi.bidali("Komando okerra");    
          }//Switch  
      }//if  
}//loop


//Ultrasoinu sentsorearen zerbitzu errutina
void oztopoa(){
   int trigger=digitalRead(2);
   if (trigger==0){ //Oztopoa gertuegi
      ARRISKUAN=true;
      wifi.bidali("Arriskuan");
      //Motorrak gelditu
      motorrak.gelditu();
   }
  else{ //Oztopoa ekidin da
    ARRISKUAN=false;
    wifi.bidali("Arriskutik kanpo");
  }
}  

//Konektatu wifi sarera eta hasi komunikazioa socketarekin
void konfiguratu_wifi(){
   wifi.AP_pass(AP_pass);
   wifi.dhcp(1);
   wifi.AP_izena(AP_izena);
   wifi.socketa_sortu("0", "0", zerb_helb, zerb_portua);
   wifi.hasi_komunikazioa();
   delay(500);
}


//GPS bidezko lokalizaziorako funtzioa
void kokatu(){
    detachInterrupt(0); //Etenak desgaitu lokalizazioa burutu bitartean
  //GPSa konfiguratu 
  int i;
  GPS.konfiguratu(); //Konfiguratu GPSa modu autonomoan
  for(i=0; i<irakurketa_kop; i++){
    lortu_balioak();  //Lortu latitudea eta longitudearen uneko balioak
  }
  GPS.konfiguratu(); //Beharrezkoa amaitutakoan berriro programa hau exekutatzea bluetooth bidezko komunikazioa hasteko
  wifi.bidali("Lokalizazioa amaitu da.");
  attachInterrupt(0, oztopoa, CHANGE);//Etenak gaitu
  //Begiratu oztoporen bat dagoen aurrean
   if (digitalRead(2)==0){ //Oztopoa gertuegi
      ARRISKUAN=true;
   }
   else{ //Oztopoa ekidin da
     ARRISKUAN=false;
   }
   BTSerial.begin(9600);//Bluetooth bidezko komunikazioa abiarazi
}

//GPSaren balioak eskuratzeko programa
void lortu_balioak(){  
  GPS.latitudea();
  GPS.latitudea_norantza();
  GPS.longitudea();
  GPS.longitudea_norantza();
}
